@@ -1,5 +1,6 @@
 ## creates an object of class "cophylo"
-## written by Liam J. Revell 2015, 2016, 2017
+## written by Liam J. Revell 2015, 2016, 2017, 2019, 2021, 2023, 2024
+
 cophylo<-function(tr1,tr2,assoc=NULL,rotate=TRUE,...){
 	if(!inherits(tr1,"phylo")||!inherits(tr2,"phylo")) 
 		stop("tr1 & tr2 should be objects of class \"phylo\".")
@@ -111,11 +112,21 @@ cophylo<-function(tr1,tr2,assoc=NULL,rotate=TRUE,...){
 
 ## called internally by plot.cophylo to plot a phylogram
 ## written by Liam J. Revell
+
 phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,...){
 	if(hasArg(pts)) pts<-list(...)$pts
 	else pts<-TRUE
 	if(hasArg(edge.col)) edge.col<-list(...)$edge.col
 	else edge.col<-rep("black",nrow(tree$edge))
+	if(hasArg(tip.lwd)) tip.lwd<-list(...)$tip.lwd
+	else tip.lwd<-1
+	if(hasArg(tip.lty)) tip.lty<-list(...)$tip.lty
+	else tip.lty<-"dotted"
+	if(hasArg(tip.len)) tip.len<-list(...)$tip.len
+	else tip.len<-0.1
+	if(pts==TRUE&&tip.len==0) tip.len<-0.1
+	if(hasArg(frame)) frame<-list(...)$frame
+	else frame<-TRUE
 	d<-if(direction=="rightwards") 1 else -1
 	## sub "_" for " "
 	tree$tip.label<-gsub("_"," ",tree$tip.label)
@@ -124,9 +135,12 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 	## rescale tree so it fits in one half of the plot
 	## with enough space for labels
 	if(ftype=="off") fsize<-0
-	sh<-fsize*strwidth(tree$tip.label)
-	tree$edge.length<-tree$edge.length/max(nodeHeights(tree))*(part-max(sh))
 	n<-Ntip(tree)
+	sh<-fsize*strwidth(tree$tip.label)
+	H<-nodeHeights(tree)
+	th<-sapply(1:n,function(i,x,e) x[which(e==i)],x=H[,2],
+		e=tree$edge[,2])+tip.len*max(H)
+	tree$edge.length<-tree$edge.length/max(th/(part-sh))
 	## reorder cladewise to assign tip positions
 	cw<-reorder(tree,"cladewise")
 	y<-vector(length=n+cw$Nnode)
@@ -158,34 +172,124 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 		segments(x0=d*xx[1:(length(xx)-1)],y0=yy[1:(length(yy)-1)],
 			x1=d*xx[2:length(xx)],y1=yy[2:length(yy)],lwd=lwd,lend=2,col=edge.col[ee])
 	}
+	h<-part-0.5-tip.len*(max(X)-min(X))-fsize*strwidth(tree$tip.label)
 	## plot links to tips
-	h<-max(X)+0.1*(max(X)-min(X))+max(fsize*strwidth(tree$tip.label))-
-		fsize*strwidth(tree$tip.label)
 	for(i in 1:n){ 
-		lines(d*c(X[which(cw$edge[,2]==i),2],h[i]),rep(y[i],2),lwd=1,lty="dotted")
+		lines(d*c(X[which(cw$edge[,2]==i),2],h[i]+tip.len*(max(X)-min(X))),rep(y[i],2),
+			lwd=tip.lwd,lty=tip.lty)
 		if(pts) points(d*X[which(cw$edge[,2]==i),2],y[i],pch=16,cex=pts*0.7*sqrt(lwd))
-	}
+	}	
 	## plot tip labels
 	font<-which(c("off","reg","b","i","bi")==ftype)-1
 	if(font>0){
-		for(i in 1:n) text(d*max(h+fsize*strwidth(tree$tip.label)),y[i],
+		for(i in 1:n) TEXTBOX(d*(h[i]+fsize*strwidth(tree$tip.label[i])+
+			tip.len*(max(X)-min(X))),y[i],
 			tree$tip.label[i], pos=if(d<0) 4 else 2,offset=0,
-			cex=fsize,font=font)
-	}
+			cex=fsize,font=font,frame=frame)
+	}	
 	PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
 		show.tip.label=if(ftype!="off") TRUE else FALSE,show.node.label=FALSE,
 		font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0,
 		x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
 		direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
-		edge=cw$edge,xx=d*sapply(1:(Ntip(cw)+cw$Nnode),
+		edge=tree$edge,xx=d*sapply(1:(Ntip(cw)+cw$Nnode),
 		function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
 	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	## return rightmost or leftmost edge of tip labels
-	invisible(d*max(h+fsize*strwidth(tree$tip.label)))
+	invisible(d*max(h+fsize*strwidth(tree$tip.label)+tip.len*(max(X)-min(X))))
 }
 
+cladogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,...){
+	if(hasArg(pts)) pts<-list(...)$pts
+	else pts<-TRUE
+	if(hasArg(edge.col)) edge.col<-list(...)$edge.col
+	else edge.col<-rep("black",nrow(tree$edge))
+	if(hasArg(tip.lwd)) tip.lwd<-list(...)$tip.lwd
+	else tip.lwd<-1
+	if(hasArg(tip.lty)) tip.lty<-list(...)$tip.lty
+	else tip.lty<-"dotted"
+	if(hasArg(tip.len)) tip.len<-list(...)$tip.len
+	else tip.len<-0.1
+	if(pts==TRUE&&tip.len==0) tip.len<-0.1
+	if(hasArg(frame)) frame<-list(...)$frame
+	else frame<-TRUE
+	d<-if(direction=="rightwards") 1 else -1
+	## sub "_" for " "
+	tree$tip.label<-gsub("_"," ",tree$tip.label)
+	## check if edge lenths
+	if(is.null(tree$edge.length)) tree<-compute.brlen(tree)
+	## rescale tree so it fits in one half of the plot
+	## with enough space for labels
+	if(ftype=="off") fsize<-0
+	n<-Ntip(tree)
+	sh<-fsize*strwidth(tree$tip.label)
+	H<-nodeHeights(tree)
+	th<-sapply(1:n,function(i,x,e) x[which(e==i)],x=H[,2],
+		e=tree$edge[,2])+tip.len*max(H)
+	tree$edge.length<-tree$edge.length/max(th/(part-sh))
+	## reorder cladewise to assign tip positions
+	cw<-reorder(tree,"cladewise")
+	y<-vector(length=n+cw$Nnode)
+	y[cw$edge[cw$edge[,2]<=n,2]]<-0:(n-1)/(n-1)
+	## reorder pruningwise for post-order traversal
+	pw<-reorder(tree,"pruningwise")
+	nn<-unique(pw$edge[,1])
+	## compute vertical position of each edge
+	for(i in 1:length(nn)){
+		desc<-pw$edge[which(pw$edge[,1]==nn[i]),2]
+		n1<-desc[which(y[desc]==min(y[desc]))]
+		n2<-desc[which(y[desc]==max(y[desc]))]
+		v1<-pw$edge.length[which(pw$edge[,2]==n1)]
+		v2<-pw$edge.length[which(pw$edge[,2]==n2)]
+		y[nn[i]]<-((1/v1)*y[n1]+(1/v2)*y[n2])/(1/v1+1/v2)
+	}
+	## compute start & end points of each edge
+	X<-nodeHeights(cw)-0.5
+	## plot horizontal edges
+	for(i in 1:nrow(X)) lines(d*X[i,],y[cw$edge[i,]],lwd=lwd,lend=2,
+		col=edge.col[i])
+	h<-part-0.5-tip.len*(max(X)-min(X))-fsize*strwidth(tree$tip.label)
+	## plot links to tips
+	for(i in 1:n){ 
+		lines(d*c(X[which(cw$edge[,2]==i),2],h[i]+tip.len*(max(X)-min(X))),rep(y[i],2),
+			lwd=tip.lwd,lty=tip.lty)
+		if(pts) points(d*X[which(cw$edge[,2]==i),2],y[i],pch=16,cex=pts*0.7*sqrt(lwd))
+	}	
+	## plot tip labels
+	font<-which(c("off","reg","b","i","bi")==ftype)-1
+	if(font>0){
+		for(i in 1:n) TEXTBOX(d*(h[i]+fsize*strwidth(tree$tip.label[i])+
+			tip.len*(max(X)-min(X))),y[i],
+			tree$tip.label[i], pos=if(d<0) 4 else 2,offset=0,
+			cex=fsize,font=font,frame=frame)
+	}	
+	PP<-list(type="cladogram",use.edge.length=TRUE,node.pos=1,
+		show.tip.label=if(ftype!="off") TRUE else FALSE,show.node.label=FALSE,
+		font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0,
+		x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
+		direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
+		edge=tree$edge,xx=d*sapply(1:(Ntip(cw)+cw$Nnode),
+		function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
+	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
+	## return rightmost or leftmost edge of tip labels
+	invisible(d*max(h+fsize*strwidth(tree$tip.label)+tip.len*(max(X)-min(X))))
+}
+
+## internally used function
+
+TEXTBOX<-function(x,y,label,pos,offset,cex,font,frame=TRUE){
+	if(frame) rect(x,y-0.5*strheight(label,cex=cex,font=font),
+		x+if(pos==4) strwidth(label,
+		cex=cex,font=font) else -strwidth(label,cex=cex,font=font),
+		y+0.5*strheight(label,cex=cex,font=font),border=FALSE,
+		col=if(par()$bg%in%c("white","transparent")) "white" else par()$bg)
+	text(x=x,y=y,label=label,pos=pos,offset=offset,cex=cex,font=font)
+}
+	
+
 ## plot links between tip taxa according to assoc
-## written by Liam J. Revell 2015, 2016
+## written by Liam J. Revell 2015, 2016, 2019
+
 makelinks<-function(obj,x,link.type="curved",link.lwd=1,link.col="black",
 	link.lty="dashed"){
 	if(length(link.lwd)==1) link.lwd<-rep(link.lwd,nrow(obj$assoc))
@@ -194,24 +298,32 @@ makelinks<-function(obj,x,link.type="curved",link.lwd=1,link.col="black",
 	for(i in 1:nrow(obj$assoc)){
 		ii<-which(obj$trees[[1]]$tip.label==obj$assoc[i,1])
 		jj<-which(obj$trees[[2]]$tip.label==obj$assoc[i,2])
-		y<-c((ii-1)/(Ntip(obj$trees[[1]])-1),(jj-1)/(Ntip(obj$trees[[2]])-1))
-		if(link.type=="straight") lines(x,y,lty=link.lty[i],
-			lwd=link.lwd[i],col=link.col[i])
-		else if(link.type=="curved") drawCurve(x,y,lty=link.lty[i],
-			lwd=link.lwd[i],col=link.col[i])
+		for(j in 1:length(ii)) for(k in 1:length(jj)){
+			y<-c((ii[j]-1)/(Ntip(obj$trees[[1]])-1),
+				(jj[k]-1)/(Ntip(obj$trees[[2]])-1))
+			if(link.type=="straight") lines(x,y,lty=link.lty[i],
+				lwd=link.lwd[i],col=link.col[i])
+			else if(link.type=="curved") drawCurve(x,y,lty=link.lty[i],
+				lwd=link.lwd[i],col=link.col[i])
+		}
 	}
 }
 
 ## plot method for class "multiCophylo"
+
 plot.multiCophylo<-function(x,...){
 	par(ask=TRUE)
 	for(i in 1:length(x)) plot.cophylo(x[[i]],...)
 }
 
 ## plot an object of class "cophylo"
-## written by Liam J. Revell 2015, 2016, 2017
+## written by Liam J. Revell 2015, 2016, 2017, 2021, 2023
+
 plot.cophylo<-function(x,...){
 	plot.new()
+	if(hasArg(type)) type<-list(...)$type
+	else type<-"phylogram"
+	if(length(type)==1) type<-rep(type,2)
 	if(hasArg(mar)) mar<-list(...)$mar
 	else mar<-c(0.1,0.1,0.1,0.1)
 	if(hasArg(xlim)) xlim<-list(...)$xlim
@@ -232,7 +344,11 @@ plot.cophylo<-function(x,...){
 	else edge.col<-list(
 		left=rep("black",nrow(x$trees[[1]]$edge)),
 		right=rep("black",nrow(x$trees[[2]]$edge)))
+	if(hasArg(frame)) frame<-list(...)$frame
+	else frame<-c(TRUE,TRUE)
+	if(length(frame)==1) frame<-rep(frame,2)
 	obj<-list(...)
+	if(is.null(obj$part)) obj$part<-0.4
 	par(mar=mar)
 	plot.window(xlim=xlim,ylim=ylim)
 	leftArgs<-rightArgs<-obj
@@ -245,30 +361,43 @@ plot.cophylo<-function(x,...){
 			sb.fsize<- if(length(obj$fsize)>2) obj$fsize[3] else 1
 		} else sb.fsize<-1
 	} else sb.fsize<-1
-	x1<-do.call("phylogram",c(list(tree=x$trees[[1]],part=0.4),leftArgs))
+	if(!is.null(obj$ftype)){
+		if(length(obj$ftype)>1){
+			leftArgs$ftype<-obj$ftype[1]
+			rightArgs$ftype<-obj$ftype[2]
+		}
+	}
+	leftArgs$frame<-frame[1]
+	rightArgs$frame<-frame[2]
+	plotter<-if(type[1]=="cladogram") "cladogram" else "phylogram"
+	x1<-do.call(plotter,c(list(tree=x$trees[[1]]),leftArgs))
+	plotter<-if(type[2]=="cladogram") "cladogram" else "phylogram"
 	left<-get("last_plot.phylo",envir=.PlotPhyloEnv)
-	x2<-do.call("phylogram",c(list(tree=x$trees[[2]],part=0.4,
-		direction="leftwards"),rightArgs))
+	x2<-do.call(plotter,c(list(tree=x$trees[[2]],direction="leftwards"),
+		rightArgs))
 	right<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 	if(!is.null(x$assoc)) makelinks(x,c(x1,x2),link.type,link.lwd,link.col,
 		link.lty)
 	else cat("No associations provided.\n")
-	if(any(scale.bar>0)) add.scalebar(x,scale.bar,sb.fsize)
 	assign("last_plot.cophylo",list(left=left,right=right),envir=.PlotPhyloEnv)
+	if(any(scale.bar>0)) add.scalebar(x,scale.bar,sb.fsize)
 }
 
 ## add scale bar
-## written by Liam J. Revell 2015
+## written by Liam J. Revell 2015, 2023
+
 add.scalebar<-function(obj,scale.bar,fsize){
 	if(scale.bar[1]>0){
-		s1<-(0.4-max(fsize*strwidth(obj$trees[[1]]$tip.label)))/max(nodeHeights(obj$trees[[1]]))
+		pp<-get("last_plot.cophylo",envir=.PlotPhyloEnv)[[1]]
+		s1<-diff(range(pp$xx))/max(nodeHeights(obj$trees[[1]]))
 		lines(c(-0.5,-0.5+scale.bar[1]*s1),rep(-0.05,2))
 		lines(rep(-0.5,2),c(-0.05,-0.06))
 		lines(rep(-0.5+scale.bar[1]*s1,2),c(-0.05,-0.06))
 		text(mean(c(-0.5,-0.5+scale.bar[1]*s1)),rep(-0.05,2),scale.bar[1],pos=1)
 	}
 	if(scale.bar[2]>0){
-		s2<-(0.4-max(fsize*strwidth(obj$trees[[2]]$tip.label)))/max(nodeHeights(obj$trees[[2]]))
+		pp<-get("last_plot.cophylo",envir=.PlotPhyloEnv)[[2]]
+		s2<-diff(range(pp$xx))/max(nodeHeights(obj$trees[[2]]))
 		lines(c(0.5-scale.bar[2]*s2,0.5),rep(-0.05,2))	
 		lines(rep(0.5-scale.bar[2]*s2,2),c(-0.05,-0.06))
 		lines(rep(0.5,2),c(-0.05,-0.06))
@@ -278,6 +407,7 @@ add.scalebar<-function(obj,scale.bar,fsize){
 
 ## print an object of class "cophylo"
 ## written by Liam J. Revell 2015
+
 print.cophylo<-function(x, ...){
     cat("Object of class \"cophylo\" containing:\n\n")
     cat("(1) 2 (possibly rotated) phylogenetic trees in an object of class \"multiPhylo\".\n\n")
@@ -285,10 +415,12 @@ print.cophylo<-function(x, ...){
 }
 
 ## print method for "multiCophylo" object
+
 print.multiCophylo<-function(x, ...)
 	cat("Object of class \"multiCophylo\" containg",length(x),"objects of class \"cophylo\".\n\n")
 
 ## written by Liam J. Revell 2015, 2016
+
 tipRotate<-function(tree,x,...){
 	if(hasArg(fn)) fn<-list(...)$fn
 	else fn<-function(x) x^2
@@ -301,7 +433,7 @@ tipRotate<-function(tree,x,...){
 	else max.exhaustive<-20
 	if(hasArg(rotate.multi)) rotate.multi<-list(...)$rotate.multi
 	else rotate.multi<-FALSE
-	if(rotate.multi) rotate.multi<-!is.binary.tree(tree)
+	if(rotate.multi) rotate.multi<-!is.binary(tree)
 	if(hasArg(anim.cophylo)) anim.cophylo<-list(...)$anim.cophylo
 	else anim.cophylo<-FALSE
 	if(anim.cophylo){
@@ -314,6 +446,8 @@ tipRotate<-function(tree,x,...){
 		if(is.null(left)&&is.null(right)) anim.cophylo<-FALSE
 		if(hasArg(only.accepted)) only.accepted<-list(...)$only.accepted
 		else only.accepted<-TRUE
+		if(hasArg(sleep)) sleep<-list(...)$sleep
+		else sleep<-0.1
 	}
 	tree<-reorder(tree)
 	nn<-1:tree$Nnode+length(tree$tip.label)
@@ -349,6 +483,7 @@ tipRotate<-function(tree,x,...){
 		else if(is.null(right)) plot(cophylo(left,tree,assoc=assoc,rotate=FALSE),...)
 		nodelabels.cophylo(node=i+Ntip(tree),pie=1,col="red",cex=0.4,
 			which=if(is.null(left)) "left" else "right")
+		Sys.sleep(sleep)
 		dev.flush()
 	}
 	if("pre"%in%methods){
@@ -398,8 +533,8 @@ tipRotate<-function(tree,x,...){
 
 ## multi2di for "multiPhylo" object
 
-MULTI2DI<-function(x){
-	obj<-lapply(x,multi2di)
+MULTI2DI<-function(x,...){
+	obj<-lapply(x,multi2di,...)
 	class(obj)<-"multiPhylo"
 	obj
 }
@@ -452,7 +587,7 @@ summary.cophylo<-function(object,...){
 		Ntip(object$trees[[2]]),"species.\n\n"))
 	cat("Association (assoc) table as follows:\n\n")
 	maxl<-max(sapply(strsplit(object$assoc[,1],""),length))
-	cat(paste("\tleft:",paste(rep(" ",maxl-5),collapse=""),
+	cat(paste("\tleft:",paste(rep(" ",max(0,maxl-5)),collapse=""),
 		"\t----\tright:\n",sep=""))
 	nulo<-apply(object$assoc,1,function(x,maxl) cat(paste("\t",x[1],
 		paste(rep(" ",maxl-length(strsplit(x[1],split="")[[1]])),
@@ -460,3 +595,8 @@ summary.cophylo<-function(object,...){
 	cat("\n")
 }
 
+## S3 Ntip, Nedge, and Nnode methods
+
+Ntip.cophylo<-function(phy) sapply(phy$trees,Ntip)
+Nnode.cophylo<-function(phy,...) sapply(phy$trees,Nnode,...)
+Nedge.cophylo<-function(phy) sapply(phy$trees,Nedge)
\ No newline at end of file
