@@ -1,7 +1,9 @@
-## function for computing phylogenetic signal by the lambda (Pagel 1999) of K (Blomberg et al. 2003) methods
-## written by Liam J. Revell 2011/2012, 2019
+## function for computing phylogenetic signal by the lambda (Pagel 1999)
+## or K (Blomberg et al. 2003) methods
+## written by Liam J. Revell 2011/2012, 2019, 2020, 2021, 2023
 
-phylosig<-function(tree,x,method="K",test=FALSE,nsim=1000,se=NULL,start=NULL,control=list()){
+phylosig<-function(tree,x,method="K",test=FALSE,nsim=1000,
+	se=NULL,start=NULL,control=list(),niter=10){
 	# some minor error checking
 	if(!inherits(tree,"phylo")) 
 		stop("tree should be an object of class \"phylo\".")
@@ -135,8 +137,13 @@ phylosig<-function(tree,x,method="K",test=FALSE,nsim=1000,se=NULL,start=NULL,con
 		x<-x[rownames(C)]
 		maxlam<-maxLambda(tree)
 		if(!me){
-			res<-optimize(f=likelihoodLambda,interval=c(0,maxlam),
-				y=x,C=C,maximum=TRUE) # optimize lambda
+			INTERVAL<-cbind(
+				seq(0,maxlam-maxlam/niter,length.out=niter),
+				seq(maxlam/niter,maxlam,length.out=niter))
+			FITS<-apply(INTERVAL,1,optimize,f=likelihoodLambda,
+				y=x,C=C,maximum=TRUE)
+			LIKS<-sapply(FITS,function(x) x$objective)
+			res<-FITS[[which(LIKS==max(LIKS))[1]]]
 			if(!test){
 				object<-list(lambda=res$maximum,logL=res$objective,
 					lik=function(lambda) likelihoodLambda(lambda,
@@ -151,14 +158,19 @@ phylosig<-function(tree,x,method="K",test=FALSE,nsim=1000,se=NULL,start=NULL,con
 					likelihoodLambda(lambda,C=C,y=x))
 			}
 		} else {
+			FITS<-list()
 			control$fnscale=-1
 			M<-M[rownames(C),colnames(C)]
-			if(is.null(start)) s<-c(0.02*runif(n=1)*mean(pic(x,
-				multi2di(tree))^2),runif(n=1))
-			else s<-start
-			res<-optim(s,likelihoodLambda.me,C=C,y=x,M=M,
-				method="L-BFGS-B",lower=c(0,0),upper=c(Inf,maxlam),
-				control=control)
+			for(i in 1:niter){
+				if(is.null(start)) s<-c(0.5*runif(n=1)*mean(pic(x,
+					multi2di(tree,random=FALSE))^2),runif(n=1))
+				else s<-start
+				FITS[[i]]<-optim(s,likelihoodLambda.me,C=C,y=x,M=M,
+					method="L-BFGS-B",lower=c(0,0),upper=c(Inf,maxlam),
+					control=control)
+			}
+			LIKS<-sapply(FITS,function(x) x$value)
+			res<-FITS[[which(LIKS==max(LIKS))[1]]]
 			if(!test){
 				object<-list(lambda=res$par[2],sig2=res$par[1],
 					logL=res$value,convergence=res$convergence,
@@ -166,9 +178,15 @@ phylosig<-function(tree,x,method="K",test=FALSE,nsim=1000,se=NULL,start=NULL,con
 					sig2=res$par[1]) likelihoodLambda.me(c(sig2,lambda),
 					C=C,M=M,y=x))
 			} else {
-				res0<-optim(c(s[1],0),likelihoodLambda.me,C=C,
-					y=x,M=M,method="L-BFGS-B",lower=c(0,0),
-					upper=c(Inf,1e-10),control=control)
+				for(i in 1:niter){
+					s<-0.5*runif(n=1)*mean(pic(x,multi2di(tree,
+						random=FALSE))^2)
+					FITS[[i]]<-optim(c(s,0),likelihoodLambda.me,C=C,
+						y=x,M=M,method="L-BFGS-B",lower=c(0,0),
+						upper=c(Inf,1e-10),control=control)
+				}
+				LIKS<-sapply(FITS,function(x) x$value)
+				res0<-FITS[[which(LIKS==max(LIKS))[1]]]
 				P<-as.numeric(pchisq(2*(res$value-res0$value),df=1,
 					lower.tail=FALSE))
 				object<-list(lambda=res$par[2],sig2=res$par[1],
@@ -229,11 +247,22 @@ plot.phylosig<-function(x,...){
 		if(attr(x,"method")=="K"&&attr(x,"test")) "K" else "sig2"
 	if(hasArg(res)) res<-list(...)$res
 	else res<-100
+	if(hasArg(las)) las<-list(...)$las
+	else las<-par()$las
+	if(hasArg(cex.lab)) cex.lab<-list(...)$cex.lab*par()$cex
+	else cex.lab<-par()$cex.lab
+	if(hasArg(cex.axis)) cex.axis<-list(...)$cex.axis*par()$cex
+	else cex.axis<-par()$cex.axis
+	if(hasArg(bty)) bty<-list(...)$bty
+	else bty<-par()$bty
+	if(hasArg(xlim)) xlim<-list(...)$xlim
+	else xlim<-NULL
 	if(attr(x,"method")=="lambda"){
 		lambda<-seq(0,max(c(1,x$lambda)),length.out=res)
 		logL<-sapply(lambda,x$lik)
 		plot(lambda,logL,xlab=expression(lambda),ylab="log(L)",
-			type="l",bty="l")
+			type="l",bty=bty,las=las,cex.lab=cex.lab,cex.axis=cex.axis,
+			xlim=xlim)
 		lines(rep(x$lambda,2),c(par()$usr[3],x$logL),lty="dotted")
 		text(x=x$lambda+0.01*diff(par()$usr[1:2]),
 			par()$usr[3]+0.5*diff(par()$usr[3:4]),
@@ -256,12 +285,13 @@ plot.phylosig<-function(x,...){
 				cat("Sorry. This is not a valid plotting option for your object.\n\n")
 			else {
 				hist(x$sim.K,breaks=min(c(max(12,round(length(x$sim.K)/10)),
-					20)),bty="l",col="lightgrey",border="lightgrey",
-					main="",xlab="K",ylab="null distribution of K")
+					20)),bty=bty,col="lightgrey",border="lightgrey",
+					main="",xlab="K",ylab="null distribution of K",
+					las=las,cex.lab=cex.lab,cex.axis=cex.axis)
 				arrows(x0=x$K,y0=par()$usr[4],y1=0,length=0.12,
 					col=make.transparent("blue",0.5),lwd=2)
 				text(x$K,0.95*par()$usr[4],"observed value of K",
-					pos=if(x$K>mean(x$sim.K)) 2 else 4)
+					pos=if(x$K>mean(range(x$sim.K))) 2 else 4)
 			}
 		} else if(what=="sig2"){
 			if(attr(x,"se")==FALSE)
@@ -270,7 +300,8 @@ plot.phylosig<-function(x,...){
 				sig2<-seq(0.25*x$sig2,1.75*x$sig2,length.out=res)
 				logL<-sapply(sig2,x$lik)
 				plot(sig2,logL,xlab=expression(sigma^2),ylab="log(L)",
-					type="l",bty="l")
+					type="l",bty=bty,las=las,cex.lab=cex.lab,
+					cex.axis=cex.axis,xlim=xlim)
 				lines(rep(x$sig2,2),c(par()$usr[3],x$logL),lty="dotted")
 				text(x=x$sig2+0.01*diff(par()$usr[1:2]),
 					par()$usr[3]+0.5*diff(par()$usr[3:4]),
