@@ -137,7 +137,7 @@ arcPhylogram<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,
 }
 
 ## function to plot simmap tree in type "phylogram"
-## written by Liam J. Revell 2011-2023
+## written by Liam J. Revell 2011-2024
 updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	add,offset,direction,setEnv,xlim,ylim,placement,tips,split.vertical,lend,
 	asp,plot,underscore){
@@ -184,6 +184,16 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 				median(Y[1:Ntip(pw)]))))
 			if(length(mm>1)) mm<-mm[which(Y[desc][mm]==min(Y[desc][mm]))]
 			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="right"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==max(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="left"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==min(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
 		}
 	}
 	# compute node heights
@@ -321,6 +331,16 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 				median(Y[1:Ntip(pw)]))))
 			if(length(mm>1)) mm<-mm[which(Y[desc][mm]==min(Y[desc][mm]))]
 			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="right"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==max(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="left"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==min(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
 		}
 	}
 	# compute node heights
@@ -549,6 +569,16 @@ plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
 				median(Y[1:Ntip(pw)]))))
 			if(length(mm>1)) mm<-mm[which(Y[desc][mm]==min(Y[desc][mm]))]
 			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="right"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==max(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
+		} else if(placement=="left"){
+			desc<-getDescendants(pw,nodes[i])
+			desc<-desc[desc<=Ntip(pw)]
+			mm<-which(Y[desc]==min(Y[desc]))
+			Y[nodes[i]]<-Y[desc][mm]
 		}
 	}
 	# compute node heights
@@ -606,7 +636,6 @@ plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
 	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 }
 
-
 ## adds legend to an open stochastic map style plot
 ## written by Liam J. Revell 2013, 2016, 2017, 2023
 add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
@@ -725,25 +754,42 @@ plot.simmap<-function(x,...) plotSimmap(x,...)
 plot.multiSimmap<-function(x,...) plotSimmap(x,...)
 
 ## function to split vertical plotted lines by the states of daughter edges
-## written by Liam J. Revell 2015
+## written by Liam J. Revell 2015, 2024
 splitEdgeColor<-function(tree,colors,lwd=2){
 	obj<-get("last_plot.phylo",envir=.PlotPhyloEnv)
-	for(i in 1:tree$Nnode+Ntip(tree)){
-		daughters<-tree$edge[which(tree$edge[,1]==i),2]
-		cols<-vector()
-		for(j in 1:length(daughters)){
-			jj<-which(tree$edge[,2]==daughters[j])
-			cols[j]<-if(tree$maps[[jj]][1]==0&&length(tree$maps[[jj]])>1) colors[names(tree$maps[[jj]])[2]] 
-				else colors[names(tree$maps[[jj]])[1]]
+	if(obj$direction%in%c("leftwards","rightwards")){
+		for(i in 1:tree$Nnode+Ntip(tree)){
+			daughters<-tree$edge[which(tree$edge[,1]==i),2]
+			cols<-vector()
+			for(j in 1:length(daughters)){
+				jj<-which(tree$edge[,2]==daughters[j])
+				cols[j]<-if(tree$maps[[jj]][1]==0&&length(tree$maps[[jj]])>1) 
+					colors[names(tree$maps[[jj]])[2]] else colors[names(tree$maps[[jj]])[1]]
+			}
+			ii<-order(obj$yy[c(i,daughters)])
+			jj<-order(obj$yy[daughters])
+			x0<-x1<-rep(obj$xx[i],length(daughters))
+			y0<-obj$yy[c(i,daughters)][ii][1:length(daughters)]
+			y1<-obj$yy[c(i,daughters)][ii][2:(length(daughters)+1)]
+			cols<-cols[jj]
+			for(j in 1:length(x0)) segments(x0[j],y0[j],x1[j],y1[j],col=cols[j],lwd=lwd,lend=2)
+		}
+	} else if(obj$direction%in%c("upwards","downwards")){
+			for(i in 1:tree$Nnode+Ntip(tree)){
+				daughters<-tree$edge[which(tree$edge[,1]==i),2]
+				cols<-vector()
+				for(j in 1:length(daughters)){
+					jj<-which(tree$edge[,2]==daughters[j])
+					cols[j]<-if(tree$maps[[jj]][1]==0&&length(tree$maps[[jj]])>1) 
+						colors[names(tree$maps[[jj]])[2]] else colors[names(tree$maps[[jj]])[1]]
+				}
+				ii<-order(obj$xx[c(i,daughters)])
+				jj<-order(obj$xx[daughters])
+				y0<-y1<-rep(obj$yy[i],length(daughters))
+				x0<-obj$xx[c(i,daughters)][ii][1:length(daughters)]
+				x1<-obj$xx[c(i,daughters)][ii][2:(length(daughters)+1)]
+				cols<-cols[jj]
+				for(j in 1:length(x0)) segments(x0[j],y0[j],x1[j],y1[j],col=cols[j],lwd=lwd,lend=2)
 		}
-		ii<-order(obj$yy[c(i,daughters)])
-		jj<-order(obj$yy[daughters])
-		x0<-x1<-rep(obj$xx[i],length(daughters))
-		y0<-obj$yy[c(i,daughters)][ii][1:length(daughters)]
-		y1<-obj$yy[c(i,daughters)][ii][2:(length(daughters)+1)]
-		cols<-cols[jj]
-		for(j in 1:length(x0)) segments(x0[j],y0[j],x1[j],y1[j],col=cols[j],lwd=lwd,lend=2)
 	}
 }
-
-
